@@ -1,41 +1,57 @@
 # encoding: utf-8
 
-"""
-Gherkin step implementations for shape-related features.
-"""
+"""Gherkin step implementations for shape-related features."""
 
-from __future__ import absolute_import, print_function
+from __future__ import (
+    absolute_import, division, print_function, unicode_literals
+)
 
 import hashlib
 
 from behave import given, when, then
 
 from pptx import Presentation
-from pptx.chart.chart import Chart
-from pptx.dml.color import RGBColor
-from pptx.enum.chart import XL_CHART_TYPE
-from pptx.enum.dml import MSO_FILL, MSO_THEME_COLOR
-from pptx.enum.shapes import (
-    MSO_CONNECTOR, MSO_SHAPE, MSO_SHAPE_TYPE, PP_MEDIA_TYPE
-)
+from pptx.enum.shapes import MSO_SHAPE, MSO_SHAPE_TYPE, PP_MEDIA_TYPE
 from pptx.action import ActionSetting
-from pptx.util import Emu, Inches
+from pptx.util import Emu
 
-from helpers import (
-    cls_qname, saved_pptx_path, test_file, test_pptx, test_text
-)
+from helpers import cls_qname, test_file, test_pptx
 
 
 # given ===================================================
 
+@given('an autoshape')
+def given_an_autoshape(context):
+    prs = Presentation(test_pptx('shp-autoshape-adjustments'))
+    context.shape = prs.slides[0].shapes[0]
+
+
+@given('an autoshape having text')
+def given_an_autoshape_having_text(context):
+    prs = Presentation(test_pptx('shp-autoshape-props'))
+    context.shape = prs.slides[0].shapes[0]
+
+
+@given("(builder._start_x, builder._start_y) is ({x_str}, {y_str})")
+def given_builder_start_x_builder_start_y_is_x_y(context, x_str, y_str):
+    builder = context.builder
+    builder._start_x, builder._start_y = int(x_str), int(y_str)
+
+
+@given("(builder._x_scale, builder._y_scale) is ({p_str}, {q_str})")
+def given_builder_x_scale_builder_y_scale_is_p_q(context, p_str, q_str):
+    builder = context.builder
+    builder._x_scale, builder._y_scale = float(p_str), float(q_str)
+
+
 @given('a chevron shape')
 def given_a_chevron_shape(context):
     prs = Presentation(test_pptx('shp-autoshape-adjustments'))
     context.shape = prs.slides[0].shapes[0]
 
 
-@given('a connector')
-def given_a_connector(context):
+@given('a Connector object as shape')
+def given_a_Connector_object_as_shape(context):
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[0]
     context.shape = sld.shapes[4]
@@ -63,29 +79,51 @@ def given_a_connector_having_its_end_point_at_x_y(context, x, y):
     context.connector = sld.shapes[0]
 
 
-@given('a graphic frame')  # shouldn't matter, but this one contains a table
-def given_a_graphic_frame(context):
+@given('an empty GroupShape object as shape')
+def given_an_empty_GroupShape_object_as_shape(context):
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[0]
-    context.shape = sld.shapes[2]
+    context.shape = sld.shapes.add_group_shape()
 
 
-@given('a graphic frame containing a chart')
-def given_a_graphic_frame_containing_a_chart(context):
+@given('a FreeformBuilder object as builder')
+def given_a_FreeformBuilder_object_as_builder(context):
+    shapes = Presentation(test_pptx('shp-freeform')).slides[0].shapes
+    builder = shapes.build_freeform()
+    context.builder = builder
+
+
+@given('a GraphicFrame object containing a chart as shape')
+def given_a_GraphicFrame_object_containing_a_chart_as_shape(context):
     prs = Presentation(test_pptx('shp-access-chart'))
     sld = prs.slides[0]
     context.shape = sld.shapes[0]
 
 
-@given('a graphic frame containing a table')
-def given_a_graphic_frame_containing_a_table(context):
+@given('a GraphicFrame object containing a table as shape')
+def given_a_GraphicFrame_object_containing_a_table_as_shape(context):
     prs = Presentation(test_pptx('shp-access-chart'))
     sld = prs.slides[1]
     context.shape = sld.shapes[0]
 
 
-@given('a group shape')
-def given_a_group_shape(context):
+@given('a GraphicFrame object as shape')
+def given_a_GraphicFrame_object_as_shape(context):
+    # shouldn't matter, but this one contains a table
+    prs = Presentation(test_pptx('shp-common-props'))
+    sld = prs.slides[0]
+    context.shape = sld.shapes[2]
+
+
+@given('a GroupShape object as group_shape')
+def given_a_GroupShape_object_as_group_shape(context):
+    prs = Presentation(test_pptx('shp-groupshape'))
+    sld = prs.slides[0]
+    context.group_shape = sld.shapes[0]
+
+
+@given('a GroupShape object as shape')
+def given_a_GroupShape_object_as_shape(context):
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[0]
     context.shape = sld.shapes[3]
@@ -97,61 +135,47 @@ def given_a_movie_shape(context):
     context.movie = prs.slides[0].shapes[0]
 
 
-@given('a picture')
-def given_a_picture(context):
+@given('a Picture object as shape')
+def given_a_Picture_object_as_shape(context):
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[0]
     context.shape = sld.shapes[1]
 
 
-@given('a rotated {shape_type}')
-def given_a_rotated_shape_type(context, shape_type):
+@given('a rotated {shape_type} object as shape')
+def given_a_rotated_shape_type_object_as_shape(context, shape_type):
     shape_idx = {
-        'shape':         0,
-        'picture':       1,
-        'graphic frame': 2,
-        'group shape':   3,
-        'connector':     4,
+        'Shape':        0,
+        'Picture':      1,
+        'GraphicFrame': 2,
+        'GroupShape':   3,
+        'Connector':    4,
     }[shape_type]
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[1]
     context.shape = sld.shapes[shape_idx]
 
 
-@given('a shape')
-def given_a_shape(context):
+@given('a Shape object as shape')
+def given_a_Shape_object_as_shape(context):
     prs = Presentation(test_pptx('shp-common-props'))
     sld = prs.slides[0]
     context.shape = sld.shapes[0]
 
 
-@given('a SlideShapes object')
-def given_a_SlideShapes_object(context):
-    prs = Presentation(test_pptx('shp-shape-access'))
-    context.shapes = prs.slides[0].shapes
-
-
-@given('a SlideShapes object containing {a_or_no} movies')
-def given_a_SlideShapes_object_containing_a_or_no_movies(context, a_or_no):
-    pptx = {
-        'one or more': 'shp-movie-props',
-        'no':          'shp-shape-access',
-    }[a_or_no]
-    prs = Presentation(test_pptx(pptx))
-    context.prs = prs
-    context.shapes = prs.slides[0].shapes
-
-
-@given('a SlideShapes object containing 6 shapes')
-def given_a_SlideShapes_object_containing_6_shapes(context):
-    prs = Presentation(test_pptx('shp-shape-access'))
-    context.shapes = prs.slides[0].shapes
-
-
-@given('a SlideShapes object having a {type} shape at offset {idx}')
-def given_a_SlideShapes_obj_having_type_shape_at_off_idx(context, type, idx):
-    prs = Presentation(test_pptx('shp-shape-access'))
-    context.shapes = prs.slides[1].shapes
+@given('a {shape_type} object on a slide as shape')
+def given_a_shape_on_a_slide(context, shape_type):
+    shape_idx = {
+        'Shape':        0,
+        'Picture':      1,
+        'GraphicFrame': 2,
+        'GroupShape':   3,
+        'Connector':    4,
+    }[shape_type]
+    prs = Presentation(test_pptx('shp-common-props'))
+    sld = prs.slides[0]
+    context.shape = sld.shapes[shape_idx]
+    context.slide = sld
 
 
 @given('a textbox')
@@ -161,57 +185,19 @@ def given_a_textbox(context):
     context.shape = sld.shapes[5]
 
 
-@given('a {shape_type} on a slide')
-def given_a_shape_on_a_slide(context, shape_type):
-    shape_idx = {
-        'shape':         0,
-        'picture':       1,
-        'graphic frame': 2,
-        'group shape':   3,
-        'connector':     4,
-    }[shape_type]
-    prs = Presentation(test_pptx('shp-common-props'))
-    sld = prs.slides[0]
-    context.shape = sld.shapes[shape_idx]
-    context.slide = sld
-
-
 @given('a shape of known position and size')
 def given_a_shape_of_known_position_and_size(context):
     prs = Presentation(test_pptx('shp-pos-and-size'))
     context.shape = prs.slides[0].shapes[0]
 
 
-@given('an autoshape')
-def given_an_autoshape(context):
-    prs = Presentation(test_pptx('shp-autoshape-adjustments'))
-    context.shape = prs.slides[0].shapes[0]
-
-
-@given('an autoshape having text')
-def given_an_autoshape_having_text(context):
-    prs = Presentation(test_pptx('shp-autoshape-props'))
-    context.shape = prs.slides[0].shapes[0]
-
-
 # when ====================================================
 
-@when("I add a text box to the slide's shape collection")
-def when_I_add_a_text_box(context):
-    shapes = context.slide.shapes
-    x, y = (Inches(1.00), Inches(2.00))
-    cx, cy = (Inches(3.00), Inches(1.00))
-    sp = shapes.add_textbox(x, y, cx, cy)
-    sp.text = test_text
-
-
-@when("I add an auto shape to the slide's shape collection")
-def when_I_add_an_auto_shape(context):
-    shapes = context.slide.shapes
-    x, y = (Inches(1.00), Inches(2.00))
-    cx, cy = (Inches(3.00), Inches(4.00))
-    sp = shapes.add_shape(MSO_SHAPE.ROUNDED_RECTANGLE, x, y, cx, cy)
-    sp.text = test_text
+@when('I add a {cx} x {cy} shape at ({x}, {y})')
+def when_I_add_a_cx_cy_shape_at_x_y(context, cx, cy, x, y):
+    context.shape.shapes.add_shape(
+        MSO_SHAPE.ROUNDED_RECTANGLE, int(x), int(y), int(cx), int(cy)
+    )
 
 
 @when("I assign 0.15 to shape.adjustments[0]")
@@ -224,6 +210,19 @@ def when_I_assign_a_string_to_shape_text(context):
     context.shape.text = u'F\xf8o\nBar'
 
 
+@when("I assign builder.convert_to_shape() to shape")
+def when_I_assign_builder_convert_to_shape_to_shape(context):
+    builder = context.builder
+    context.shape = builder.convert_to_shape()
+
+
+@when("I assign builder.convert_to_shape({x_str}, {y_str}) to shape")
+def when_I_assign_builder_convert_to_shape_origin_x_y(context, x_str, y_str):
+    builder = context.builder
+    origin_x, origin_y = int(x_str), int(y_str)
+    context.shape = builder.convert_to_shape(origin_x, origin_y)
+
+
 @when('I assign {value} to connector.begin_x')
 def when_I_assign_value_to_connector_begin_x(context, value):
     context.connector.begin_x = int(value)
@@ -244,16 +243,42 @@ def when_I_assign_value_to_connector_end_y(context, value):
     context.connector.end_y = int(value)
 
 
-@when("I assign '{value}' to shape.name")
+@when('I assign {value} to shape.height')
+def when_I_assign_value_to_shape_height(context, value):
+    context.shape.height = int(value)
+
+
+@when('I assign {value} to shape.left')
+def when_I_assign_value_to_shape_left(context, value):
+    context.shape.left = int(value)
+
+
+@when('I assign \'{value}\' to shape.name')
 def when_I_assign_value_to_shape_name(context, value):
     context.shape.name = value
 
 
-@when("I assign {value} to shape.rotation")
+@when('I assign {value} to shape.rotation')
 def when_I_assign_value_to_shape_rotation(context, value):
     context.shape.rotation = float(value)
 
 
+@when('I assign {value} to shape.top')
+def when_I_assign_value_to_shape_top(context, value):
+    context.shape.top = int(value)
+
+
+@when('I assign {value} to shape.width')
+def when_I_assign_value_to_shape_width(context, value):
+    context.shape.width = int(value)
+
+
+@when('I call builder.add_line_segments([(100, 25), (25, 100)])')
+def when_I_call_builder_add_line_segments_100_25_25_100(context):
+    builder = context.builder
+    builder.add_line_segments([(100, 25), (25, 100)])
+
+
 @when('I call connector.begin_connect(picture, 3)')
 def when_I_call_connector_begin_connect_picture_3(context):
     connector, picture = context.connector, context.picture
@@ -266,91 +291,52 @@ def when_I_call_connector_end_connect_picture_3(context):
     connector.end_connect(picture, 3)
 
 
-@when('I call shapes.add_chart({type_}, chart_data)')
-def when_I_call_shapes_add_chart(context, type_):
-    chart_type = getattr(XL_CHART_TYPE, type_)
-    context.chart = context.shapes.add_chart(
-        chart_type, 0, 0, 0, 0, context.chart_data
-    ).chart
-
-
-@when('I call shapes.add_connector(MSO_CONNECTOR.STRAIGHT, 1, 2, 3, 4)')
-def when_I_call_shapes_add_connector(context):
-    context.connector = context.shapes.add_connector(
-        MSO_CONNECTOR.STRAIGHT, 1, 2, 3, 4
-    )
-
-
-@when('I call shapes.add_movie(file, x, y, cx, cy, poster_frame)')
-def when_I_call_shapes_add_movie(context):
-    shapes = context.shapes
-    x, y, cx, cy = Emu(2590800), Emu(571500), Emu(3962400), Emu(5715000)
-    context.movie = shapes.add_movie(
-        test_file('just-two-mice.mp4'), x, y, cx, cy,
-        test_file('just-two-mice.png')
-    )
-
-
-@when("I change the left and top of the {shape_type}")
-def when_I_change_the_position_of_the_shape(context, shape_type):
-    left, top = {
-        'shape':          (692696, 1339552),
-        'picture':       (1835696, 2711152),
-        'graphic frame': (2978696, 4082752),
-        'group shape':   (4121696, 5454352),
-        'connector':     (5264696, 6825952),
-    }[shape_type]
-    shape = context.shape
-    shape.left = left
-    shape.top = top
-
-
-@when("I change the width and height of the {shape_type}")
-def when_I_change_the_size_of_the_shape(context, shape_type):
-    width, height = {
-        'shape':          (692696, 1339552),
-        'picture':       (1835696, 2711152),
-        'graphic frame': (2978696, 4082752),
-        'group shape':   (4121696, 5454352),
-        'connector':     (5264696, 6825952),
-    }[shape_type]
-    shape = context.shape
-    shape.width = width
-    shape.height = height
-
-
-@when("I get the chart from its graphic frame")
-def when_I_get_the_chart_from_its_graphic_frame(context):
-    context.chart = context.shape.chart
-
-
-@when("I set the fill type to background")
-def when_set_fill_type_to_background(context):
-    context.shape.fill.background()
-
-
-@when("I set the fill type to solid")
-def when_set_fill_type_to_solid(context):
-    context.shape.fill.solid()
-
-
-@when("I set the foreground color brightness to 0.5")
-def when_set_fore_color_brightness_to_value(context):
-    context.shape.fill.fore_color.brightness = 0.5
-
-
-@when("I set the foreground color to a theme color")
-def when_set_fore_color_to_theme_color(context):
-    context.shape.fill.fore_color.theme_color = MSO_THEME_COLOR.ACCENT_6
-
-
-@when("I set the foreground color to an RGB value")
-def when_set_fore_color_to_RGB_value(context):
-    context.shape.fill.fore_color.rgb = RGBColor(0x12, 0x34, 0x56)
-
-
 # then ====================================================
 
+@then("accessing shape.click_action raises TypeError")
+def then_accessing_shape_click_action_raises_TypeError(context):
+    try:
+        context.shape.click_action
+    except TypeError:
+        return
+    except Exception as e:
+        raise AssertionError(
+            'Accessing GroupShape.click_action raised %s' % type(e).__name__
+        )
+    raise AssertionError('Accessing GroupShape.click_action did not raise')
+
+
+@then("builder is a FreeformBuilder object")
+def then_builder_is_a_FreeformBuilder_object(context):
+    builder = context.builder
+    class_name = builder.__class__.__name__
+    expected_value = 'FreeformBuilder'
+    assert class_name == expected_value, (
+        'Expected class name \'%s\', got \'%s\'' %
+        (expected_value, class_name)
+    )
+
+
+@then("(builder._start_x, builder._start_y) is ({x_str}, {y_str})")
+def then_builder_start_x_builder_start_y_is_x_y(context, x_str, y_str):
+    builder = context.builder
+    actual_value = builder._start_x, builder._start_y
+    expected_value = int(x_str), int(y_str)
+    assert actual_value == expected_value, (
+        'Expected %s, got %s' % (expected_value, actual_value)
+    )
+
+
+@then("(builder._x_scale, builder._y_scale) is ({p_str}, {q_str})")
+def then_builder_x_scale_builder_y_scale_is_x_y(context, p_str, q_str):
+    builder = context.builder
+    actual_value = builder._x_scale, builder._y_scale
+    expected_value = float(p_str), float(q_str)
+    assert actual_value == expected_value, (
+        'Expected %s, got %s' % (expected_value, actual_value)
+    )
+
+
 @then('connector is a Connector object')
 def then_connector_is_a_Connector_object(context):
     assert type(context.connector).__name__ == 'Connector'
@@ -404,6 +390,12 @@ def then_connector_end_y_is_an_Emu_object_with_value_y(context, y):
     assert end_y == int(y)
 
 
+@then('group_shape.shapes is a GroupShapes object')
+def then_group_shape_shapes_is_a_GroupShapes_object(context):
+    class_name = context.group_shape.shapes.__class__.__name__
+    assert class_name == 'GroupShapes', 'got %s' % class_name
+
+
 @then('movie is a Movie object')
 def then_movie_is_a_Movie_object(context):
     class_name = context.movie.__class__.__name__
@@ -456,11 +448,36 @@ def then_shape_adjustments_is_value(context):
     assert shape.adjustments[0] == 0.15
 
 
+@then('shape.chart is a Chart object')
+def then_shape_chart_is_a_Chart_object(context):
+    chart = context.shape.chart
+    class_name = chart.__class__.__name__
+    assert class_name == 'Chart', 'got %s' % class_name
+
+
 @then("shape.click_action is an ActionSetting object")
 def then_shape_click_action_is_an_ActionSetting_object(context):
     assert isinstance(context.shape.click_action, ActionSetting)
 
 
+@then('shape.has_chart is {value}')
+def then_shape_has_chart_is_value(context, value):
+    expected_value = {'True': True, 'False': False}[value]
+    actual_value = context.shape.has_chart
+    assert actual_value is expected_value, (
+        'shape.has_chart is %s' % actual_value
+    )
+
+
+@then('shape.has_table is {value}')
+def then_shape_has_table_is_value(context, value):
+    expected_value = {'True': True, 'False': False}[value]
+    actual_value = context.shape.has_table
+    assert actual_value is expected_value, (
+        'shape.has_table is %s' % actual_value
+    )
+
+
 @then('shape.has_text_frame is {value_str}')
 def then_shape_has_text_frame_is(context, value_str):
     expected_value = {'True': True, 'False': False}[value_str]
@@ -468,6 +485,22 @@ def then_shape_has_text_frame_is(context, value_str):
     assert has_text_frame is expected_value, 'got %s' % has_text_frame
 
 
+@then('shape.height == {value}')
+def then_shape_height_eq_value(context, value):
+    expected_height = int(value)
+    actual_height = context.shape.height
+    assert actual_height == expected_height, (
+        'shape.height == %s' % actual_height
+    )
+
+
+@then('shape.left == {value}')
+def then_shape_left_eq_value(context, value):
+    expected_left = int(value)
+    actual_left = context.shape.left
+    assert actual_left == expected_left, 'shape.left == %s' % actual_left
+
+
 @then('shape.line is a LineFormat object')
 def then_shape_line_is_a_LineFormat_object(context):
     shape = context.shape
@@ -479,20 +512,29 @@ def then_shape_line_is_a_LineFormat_object(context):
     )
 
 
-@then("shape.name is '{expected_value}'")
-def then_shape_name_is_value(context, expected_value):
+@then("shape.name == '{expected_value}'")
+def then_shape_name_eq_value(context, expected_value):
     shape = context.shape
     msg = "expected shape name '%s', got '%s'" % (shape.name, expected_value)
     assert shape.name == expected_value, msg
 
 
-@then('shape.part is the SlidePart of the shape')
-def then_shape_part_is_the_SlidePart_of_the_shape(context):
+@then('shape.part is a SlidePart object')
+def then_shape_part_is_a_SlidePart_object(context):
+    cls_name = type(context.shape.part).__name__
+    expected_cls_name = 'SlidePart'
+    assert cls_name == expected_cls_name, (
+        "expected '%s', got '%s'" % (expected_cls_name, cls_name)
+    )
+
+
+@then('shape.part is slide.part')
+def then_shape_part_is_slide_part(context):
     assert context.shape.part is context.slide.part
 
 
-@then("shape.rotation is {value}")
-def then_shape_rotation_is_value(context, value):
+@then("shape.rotation == {value}")
+def then_shape_rotation_eq_value(context, value):
     shape = context.shape
     expected_value = float(value)
     assert shape.rotation == expected_value, 'got %s' % expected_value
@@ -505,6 +547,15 @@ def then_shape_shape_id_equals(context, value_str):
     assert shape_id == expected_value, 'got %s' % shape_id
 
 
+@then('shape.shape_type == MSO_SHAPE_TYPE.{member_name}')
+def then_shape_shape_type_is_MSO_SHAPE_TYPE_member(context, member_name):
+    expected_shape_type = getattr(MSO_SHAPE_TYPE, member_name)
+    actual_shape_type = context.shape.shape_type
+    assert actual_shape_type == expected_shape_type, (
+        'shape.shape_type == %s' % actual_shape_type
+    )
+
+
 @then('shape.text is the string I assigned')
 def then_shape_text_is_the_string_I_assigned(context):
     shape = context.shape
@@ -517,108 +568,15 @@ def then_shape_text_is_the_text_in_the_shape(context):
     assert shape.text == u'Fee Fi\nF\xf8\xf8 Fum\nI am a shape\nwith textium'
 
 
-@then('the auto shape appears in the slide')
-def then_auto_shape_appears_in_slide(context):
-    prs = Presentation(saved_pptx_path)
-    sp = prs.slides[0].shapes[0]
-    sp_text = sp.text_frame.paragraphs[0].runs[0].text
-    assert sp.shape_type == MSO_SHAPE_TYPE.AUTO_SHAPE
-    assert sp.auto_shape_type == MSO_SHAPE.ROUNDED_RECTANGLE
-    assert sp_text == test_text
+@then('shape.top == {value}')
+def then_shape_top_eq_value(context, value):
+    expected_top = int(value)
+    actual_top = context.shape.top
+    assert actual_top == expected_top, 'shape.top == %s' % actual_top
 
 
-@then('the fill type of the shape is background')
-def then_fill_type_is_background(context):
-    assert context.shape.fill.type == MSO_FILL.BACKGROUND
-
-
-@then('the foreground color brightness of the shape is 0.5')
-def then_fore_color_brightness_is_value(context):
-    assert context.shape.fill.fore_color.brightness == 0.5
-
-
-@then('the foreground color of the shape is the RGB value I set')
-def then_fore_color_is_RGB_value_I_set(context):
-    assert context.shape.fill.fore_color.rgb == RGBColor(0x12, 0x34, 0x56)
-
-
-@then('the foreground color of the shape is the theme color I set')
-def then_fore_color_is_theme_color_I_set(context):
-    fore_color = context.shape.fill.fore_color
-    assert fore_color.theme_color == MSO_THEME_COLOR.ACCENT_6
-
-
-@then('the chart is a Chart object')
-def then_the_chart_is_a_Chart_object(context):
-    assert isinstance(context.chart, Chart)
-
-
-@then('the left and top of the {shape_type} match their new values')
-def then_left_and_top_of_shape_match_new_values(context, shape_type):
-    expected_left, expected_top = {
-        'shape':          (692696, 1339552),
-        'picture':       (1835696, 2711152),
-        'graphic frame': (2978696, 4082752),
-        'group shape':   (4121696, 5454352),
-        'connector':     (5264696, 6825952),
-    }[shape_type]
-    shape = context.shape
-    assert shape.left == expected_left, 'got left: %s' % shape.left
-    assert shape.top == expected_top, 'got top: %s' % shape.top
-
-
-@then('the left and top of the {shape_type} match their known values')
-def then_left_and_top_of_shape_match_known_values(context, shape_type):
-    expected_left, expected_top = {
-        'shape':         (1339552,  692696),
-        'picture':       (2711152, 1835696),
-        'graphic frame': (4082752, 2978696),
-        'group shape':   (5454352, 4121696),
-        'connector':     (6825952, 5264696),
-    }[shape_type]
-    shape = context.shape
-    assert shape.left == expected_left, 'got left: %s' % shape.left
-    assert shape.top == expected_top, 'got top: %s' % shape.top
-
-
-@then('the shape {has_or_not} a chart')
-def then_the_shape_has_or_not_a_chart(context, has_or_not):
-    expected_bool = {'has': True, 'does not have': False}[has_or_not]
-    shape = context.shape
-    assert shape.has_chart is expected_bool
-
-
-@then('the width and height of the {shape_type} match their known values')
-def then_width_and_height_of_shape_match_known_values(context, shape_type):
-    expected_width, expected_height = {
-        'shape':         (928192, 914400),
-        'picture':       (914400, 945232),
-        'graphic frame': (993304, 914400),
-        'group shape':   (914400, 914400),
-        'connector':     (986408, 828600),
-    }[shape_type]
-    shape = context.shape
-    assert shape.width == expected_width, 'got width: %s' % shape.width
-    assert shape.height == expected_height, 'got height: %s' % shape.height
-
-
-@then('the width and height of the {shape_type} match their new values')
-def then_width_and_height_of_shape_match_new_values(context, shape_type):
-    expected_width, expected_height = {
-        'shape':          (692696, 1339552),
-        'picture':       (1835696, 2711152),
-        'graphic frame': (2978696, 4082752),
-        'group shape':   (4121696, 5454352),
-        'connector':     (5264696, 6825952),
-    }[shape_type]
-    shape = context.shape
-    assert shape.width == expected_width, 'got width: %s' % shape.width
-    assert shape.height == expected_height, 'got height: %s' % shape.height
-
-
-@then('the text box appears in the slide')
-def then_text_box_appears_in_slide(context):
-    prs = Presentation(saved_pptx_path)
-    textbox = prs.slides[0].shapes[0]
-    textbox_text = textbox.text_frame.paragraphs[0].runs[0].text
-    assert textbox_text == test_text
+@then('shape.width == {value}')
+def then_shape_width_eq_value(context, value):
+    expected_width = int(value)
+    actual_width = context.shape.width
+    assert actual_width == expected_width, 'shape.width == %s' % actual_width
