@@ -2,7 +2,7 @@
 
 """Initialization module for python-pptx package."""
 
-__version__ = '0.6.6'
+__version__ = '0.6.8'
 
 
 import pptx.exc as exceptions
@@ -12,14 +12,14 @@ del sys
 
 from pptx.api import Presentation  # noqa
 
-from pptx.opc.constants import CONTENT_TYPE as CT
-from pptx.opc.package import PartFactory
-from pptx.parts.chart import ChartPart
-from pptx.parts.coreprops import CorePropertiesPart
-from pptx.parts.image import ImagePart
-from pptx.parts.media import MediaPart
-from pptx.parts.presentation import PresentationPart
-from pptx.parts.slide import (
+from pptx.opc.constants import CONTENT_TYPE as CT  # noqa: E402
+from pptx.opc.package import PartFactory  # noqa: E402
+from pptx.parts.chart import ChartPart  # noqa: E402
+from pptx.parts.coreprops import CorePropertiesPart  # noqa: E402
+from pptx.parts.image import ImagePart  # noqa: E402
+from pptx.parts.media import MediaPart  # noqa: E402
+from pptx.parts.presentation import PresentationPart  # noqa: E402
+from pptx.parts.slide import (  # noqa: E402
     NotesMasterPart, NotesSlidePart, SlideLayoutPart, SlideMasterPart,
     SlidePart
 )
