@@ -8,8 +8,10 @@ type in the associated XML schema.
 
 from __future__ import absolute_import, print_function
 
-from ..exc import InvalidXmlError
-from ..util import Centipoints, Emu
+import numbers
+
+from pptx.exc import InvalidXmlError
+from pptx.util import Centipoints, Emu
 
 
 class BaseSimpleType(object):
@@ -36,9 +38,9 @@ class BaseSimpleType(object):
 
     @classmethod
     def validate_int(cls, value):
-        if not isinstance(value, int):
+        if not isinstance(value, numbers.Integral):
             raise TypeError(
-                "value must be <type 'int'>, got %s" % type(value)
+                "value must be an integral type, got %s" % type(value)
             )
 
     @classmethod
@@ -292,6 +294,22 @@ class ST_BarDir(XsdStringEnumeration):
     _members = (BAR, COL)
 
 
+class ST_BubbleScale(BaseIntType):
+    """
+    String value is an integer in range 0-300, representing a percent,
+    optionally including a '%' suffix.
+    """
+    @classmethod
+    def convert_from_xml(cls, str_value):
+        if '%' in str_value:
+            return cls.convert_from_percent_literal(str_value)
+        return super(ST_BubbleScale, cls).convert_from_xml(str_value)
+
+    @classmethod
+    def validate(cls, value):
+        cls.validate_int_in_range(value, 0, 300)
+
+
 class ST_ContentType(XsdString):
     """
     Has a pretty wicked regular expression it needs to match in the schema,
@@ -477,6 +495,13 @@ class ST_LineWidth(XsdInt):
             )
 
 
+class ST_MarkerSize(XsdUnsignedByte):
+
+    @classmethod
+    def validate(cls, value):
+        cls.validate_int_in_range(value, 2, 72)
+
+
 class ST_Overlap(BaseIntType):
     """
     String value is an integer in range -100..100, representing a percent,
@@ -502,14 +527,20 @@ class ST_Percentage(BaseIntType):
     def convert_from_xml(cls, str_value):
         if '%' in str_value:
             return cls._convert_from_percent_literal(str_value)
-        return int(str_value)
+        return int(str_value) / 100000.0
+
+    @classmethod
+    def convert_to_xml(cls, value):
+        return str(int(round(value*100000.0)))
+
+    @classmethod
+    def validate(cls, value):
+        cls.validate_float_in_range(value, -21474.83648, 21474.83647)
 
     @classmethod
     def _convert_from_percent_literal(cls, str_value):
         float_part = str_value[:-1]  # trim off '%' character
-        percent_value = float(float_part)
-        int_value = int(round(percent_value * 1000))
-        return int_value
+        return float(float_part) / 100.0
 
 
 class ST_PlaceholderSize(XsdTokenEnumeration):
@@ -530,6 +561,10 @@ class ST_PositiveCoordinate(XsdLong):
         cls.validate_int_in_range(value, 0, 27273042316900)
 
 
+class ST_RelationshipId(XsdString):
+    pass
+
+
 class ST_SlideId(XsdUnsignedInt):
 
     @classmethod
