@@ -23,6 +23,21 @@ class BaseShape(object):
         self._element = shape_elm
         self._parent = parent
 
+    def __eq__(self, other):
+        """|True| if this shape object proxies the same element as *other*.
+
+        Equality for proxy objects is defined as referring to the same XML
+        element, whether or not they are the same proxy object instance.
+        """
+        if not isinstance(other, BaseShape):
+            return False
+        return self._element is other._element
+
+    def __ne__(self, other):
+        if not isinstance(other, BaseShape):
+            return True
+        return self._element is not other._element
+
     @lazyproperty
     def click_action(self):
         """
@@ -35,9 +50,11 @@ class BaseShape(object):
 
     @property
     def element(self):
-        """
-        Reference to the lxml element for this shape, e.g. a CT_Shape
-        instance.
+        """`lxml` element for this shape, e.g. a CT_Shape instance.
+
+        Note that manipulating this element improperly can produce an invalid
+        presentation file. Make sure you know what you're doing if you use
+        this to change the underlying XML.
         """
         return self._element
 
@@ -116,9 +133,11 @@ class BaseShape(object):
 
     @property
     def part(self):
-        """
-        The package part containing this object, a _BaseSlide subclass in
-        this case.
+        """The package part containing this shape.
+
+        A |BaseSlidePart| subclass in this case. Access to a slide part
+        should only be required if you are extending the behavior of |pp| API
+        objects.
         """
         return self._parent.part
 
